@@ -1,6 +1,9 @@
 from smbus2 import SMBus
 import time
 
+# Library version
+LIB_VERSION = "1.2.1"
+
 # ADS1x15 default i2c address
 I2C_address    = 0x48
 
@@ -24,45 +27,46 @@ class ADS1x15:
     INPUT_SINGLE_3 = 7
 
     # Programmable gain amplifier configuration
-    GAIN_6_144V = 0
-    GAIN_4_096V = 1
-    GAIN_2_048V = 2
-    GAIN_1_024V = 3
-    GAIN_0_512V = 4
-    GAIN_0_256V = 5
+    PGA_6_144V = 0
+    PGA_4_096V = 1
+    PGA_2_048V = 2
+    PGA_1_024V = 4
+    PGA_0_512V = 8
+    PGA_0_256V = 16
 
     # Device operating mode configuration
-    MODE_CONTINUE = 0
+    MODE_CONTINUOUS = 0
     MODE_SINGLE   = 1
+    INVALID_MODE = -1
 
     # Data rate configuration
-    DR_ADS1015_128  = 0
-    DR_ADS1015_250  = 1
-    DR_ADS1015_490  = 2
-    DR_ADS1015_920  = 3
-    DR_ADS1015_1600 = 4
-    DR_ADS1015_2400 = 5
-    DR_ADS1015_3300 = 6
-    DR_ADS1115_8    = 0
-    DR_ADS1115_16   = 1
-    DR_ADS1115_32   = 2
-    DR_ADS1115_64   = 3
-    DR_ADS1115_128  = 4
-    DR_ADS1115_250  = 5
-    DR_ADS1115_475  = 6
-    DR_ADS1115_860  = 7
+    DR_ADS101X_128  = 0
+    DR_ADS101X_250  = 1
+    DR_ADS101X_490  = 2
+    DR_ADS101X_920  = 3
+    DR_ADS101X_1600 = 4
+    DR_ADS101X_2400 = 5
+    DR_ADS101X_3300 = 6
+    DR_ADS111X_8    = 0
+    DR_ADS111X_16   = 1
+    DR_ADS111X_32   = 2
+    DR_ADS111X_64   = 3
+    DR_ADS111X_128  = 4
+    DR_ADS111X_250  = 5
+    DR_ADS111X_475  = 6
+    DR_ADS111X_860  = 7
 
     # Comparator configuration
     COMP_MODE_TRADITIONAL = 0
     COMP_MODE_WINDOW      = 1
-    COMP_POL_LOW          = 0
-    COMP_POL_HIGH         = 1
-    COMP_LAT_LATCH        = 0
-    COMP_LAT_NON_LATCH    = 1
-    COMP_QUE_ONE          = 0
-    COMP_QUE_TWO          = 1
-    COMP_QUE_FOUR         = 2
-    COMP_QUE_DISABLE      = 3
+    COMP_POL_ACTIV_LOW    = 0
+    COMP_POL_ACTIV_HIGH   = 1
+    COMP_LATCH            = 0
+    COMP_NON_LATCH        = 1
+    COMP_QUE_1_CONV       = 0
+    COMP_QUE_2_CONV       = 1
+    COMP_QUE_4_CONV       = 2
+    COMP_QUE_NONE         = 3
 
     # I2C object from smbus library
     i2c = SMBus(1)
@@ -83,26 +87,26 @@ class ADS1x15:
     _adcBits = 16
 
     # Constructor with SMBus ID and I2C address input
-    def __init__(self, busId, address = I2C_address) :
+    def __init__(self, busId: int, address: int = I2C_address) :
         self.i2c = SMBus(busId)
         self._address = address
         # Store initial config resgister to config property
         self._config = self.readRegister(self.CONFIG_REG)
 
     # Write 16-bit integer to an address pointer register
-    def writeRegister(self, address, value) :
+    def writeRegister(self, address: int, value) :
         registerValue = [(value >> 8) & 0xFF, value & 0xFF]
         self.i2c.write_i2c_block_data(self._address, address, registerValue)
 
     # Read 16-bit integer value from an address pointer register
-    def readRegister(self, address) :
+    def readRegister(self, address: int) :
         registerValue = self.i2c.read_i2c_block_data(self._address, address, 2)
         return (registerValue[0] << 8) + registerValue[1]
 
     # Set input multiplexer configuration
-    def setInput(self, input) :
+    def setInput(self, input: int) :
         # Filter input argument
-        if input < 0 or input > 7 : inputRegister = 0x0200
+        if input < 0 or input > 7 : inputRegister = 0x0000
         else : inputRegister = input << 12
         # Masking input argument bits (bit 12-14) to config register
         self._config = (self._config & 0x8FFF) | inputRegister
@@ -113,20 +117,30 @@ class ADS1x15:
         return (self._config & 0x7000) >> 12
 
     # Set programmable gain amplifier configuration
-    def setGain(self, gain) :
+    def setGain(self, gain: int) :
         # Filter gain argument
-        if gain < 0 or gain > 7 : gainRegister = 0x0200
-        else : gainRegister = gain << 9
+        if gain == self.PGA_4_096V : gainRegister = 0x0200
+        elif gain == self.PGA_2_048V : gainRegister = 0x0400
+        elif gain == self.PGA_1_024V : gainRegister = 0x0600
+        elif gain == self.PGA_0_512V : gainRegister = 0x0800
+        elif gain == self.PGA_0_256V : gainRegister = 0x0A00
+        else : gainRegister = 0x0000
         # Masking gain argument bits (bit 9-11) to config register
         self._config = (self._config & 0xF1FF) | gainRegister
         self.writeRegister(self.CONFIG_REG, self._config)
 
     # Get programmable gain amplifier configuration
     def getGain(self) :
-        return (self._config & 0x0E00) >> 9
+        gainRegister = self._config & 0x0E00
+        if gainRegister == 0x0200 : return self.PGA_4_096V
+        elif gainRegister == 0x0400 : return self.PGA_2_048V
+        elif gainRegister == 0x0600 : return self.PGA_1_024V
+        elif gainRegister == 0x0800 : return self.PGA_0_512V
+        elif gainRegister == 0x0A00 : return self.PGA_0_256V
+        else : return 0x0000
 
     # Set device operating mode configuration
-    def setMode(self, mode) :
+    def setMode(self, mode: int) :
         # Filter mode argument
         if mode == 0 : modeRegister = 0x0000
         else : modeRegister = 0x0100
@@ -139,7 +153,7 @@ class ADS1x15:
         return (self._config & 0x0100) >> 8
 
     # Set data rate configuration
-    def setDataRate(self, dataRate) :
+    def setDataRate(self, dataRate: int) :
         # Filter dataRate argument
         if dataRate < 0 or dataRate > 7 : dataRateRegister = 0x0080
         else : dataRateRegister = dataRate << 5
@@ -152,7 +166,7 @@ class ADS1x15:
         return (self._config & 0x00E0) >> 5
 
     # Set comparator mode configuration
-    def setComparatorMode(self, comparatorMode) :
+    def setComparatorMode(self, comparatorMode: int) :
         # Filter comparatorMode argument
         if comparatorMode == 1 : comparatorModeRegister = 0x0010
         else : comparatorModeRegister = 0x0000
@@ -165,7 +179,7 @@ class ADS1x15:
         return (self._config & 0x0010) >> 4
 
     # Set comparator polarity configuration
-    def setComparatorPolarity(self, comparatorPolarity) :
+    def setComparatorPolarity(self, comparatorPolarity: int) :
         # Filter comparatorPolarity argument
         if comparatorPolarity == 1 : comparatorPolarityRegister = 0x0008
         else : comparatorPolarityRegister = 0x0000
@@ -178,7 +192,7 @@ class ADS1x15:
         return (self._config & 0x0008) >> 3
 
     # Set comparator polarity configuration
-    def setComparatorLatch(self, comparatorLatch) :
+    def setComparatorLatch(self, comparatorLatch: int) :
         # Filter comparatorLatch argument
         if comparatorLatch == 1 : comparatorLatchRegister = 0x0004
         else : comparatorLatchRegister = 0x0000
@@ -191,7 +205,7 @@ class ADS1x15:
         return (self._config & 0x0004) >> 2
 
     # Set comparator queue configuration
-    def setComparatorQueue(self, comparatorQueue) :
+    def setComparatorQueue(self, comparatorQueue: int) :
         # Filter comparatorQueue argument
         if comparatorQueue < 0 or comparatorQueue > 3 : comparatorQueueRegister = 0x0002
         else : comparatorQueueRegister = comparatorQueue
@@ -204,8 +218,8 @@ class ADS1x15:
         return (self._config & 0x0003)
 
     # Set low threshold for voltage comparator
-    def setComparatorThresholdLow(self, threshold) :
-        self.writeRegister(self.LO_THRESH_REG, threshold)
+    def setComparatorThresholdLow(self, threshold: float) :
+        self.writeRegister(self.LO_THRESH_REG, round(threshold))
 
     # Get voltage comparator low threshold
     def getComparatorThresholdLow(self) :
@@ -214,8 +228,8 @@ class ADS1x15:
         return threshold
 
     # Set high threshold for voltage comparator
-    def setComparatorThresholdHigh(self, threshold) :
-        self.writeRegister(self.HI_THRESH_REG, threshold)
+    def setComparatorThresholdHigh(self, threshold: float) :
+        self.writeRegister(self.HI_THRESH_REG, round(threshold))
 
     # Get voltage comparator high threshold
     def getComparatorThresholdHigh(self) :
@@ -226,7 +240,10 @@ class ADS1x15:
     # Check if device currently not performing conversion
     def isReady(self) :
         value = self.readRegister(self.CONFIG_REG)
-        return (value & 0x8000)
+        return bool(value & 0x8000)
+
+    def isBusy(self) :
+        return not self.isReady()
 
     # Private method for starting a single-shot conversion
     def _requestADC(self, input) :
@@ -236,11 +253,17 @@ class ADS1x15:
             self.writeRegister(self.CONFIG_REG, self._config | 0x8000)
 
     # Get ADC value with current configuration
-    def getADC(self) :
+    def _getADC(self) -> int :
         t = time.time()
-        # Wait conversion process finish or reach timeout
-        while not self.isReady() and (time.time() - t) < self._conversionDelay :
-            pass
+        isContinuos = not (self._config & 0x0100)
+        # Wait conversion process finish or reach conversion time for continuous mode
+        while not self.isReady() :
+            if ((time.time() - t) * 1000) > self._conversionDelay and isContinuos :
+                break
+        return self.getValue()
+
+    # Get ADC value
+    def getValue(self) -> int :
         value = self.readRegister(self.CONVERSION_REG)
         # Shift bit based on ADC bits and change 2'complement negative value to negative integer
         value = value >> (16 - self._adcBits)
@@ -248,23 +271,24 @@ class ADS1x15:
         return value
 
     # Request single-shot conversion of a pin to ground
-    def requestADC(self, pin) :
-        if (pin >= self._maxPorts or pin < 0) : return 0
-        self._requestADC((pin << 12) + 4)
+    def requestADC(self, pin: int) :
+        if (pin >= self._maxPorts or pin < 0) : return
+        self._requestADC(pin + 4)
 
     # Get ADC value of a pin
-    def readADC(self, pin) :
+    def readADC(self, pin: int) :
+        if (pin >= self._maxPorts or pin < 0) : return 0
         self.requestADC(pin)
-        return self.getADC()
+        return self._getADC()
 
     # Request single-shot conversion between pin 0 and pin 1
     def requestADC_Differential_0_1(self) :
-        self._requestADC(0x0000)
+        self._requestADC(0)
 
     # Get ADC value between pin 0 and pin 1
     def readADC_Differential_0_1(self) :
         self.requestADC_Differential_0_1()
-        return self.getADC()
+        return self._getADC()
 
     # Get maximum voltage conversion range
     def getMaxVoltage(self) :
@@ -276,7 +300,7 @@ class ADS1x15:
         else : return 0.256
 
     # Transform an ADC value to nominal voltage
-    def toVoltage(self, value) :
+    def toVoltage(self, value: int = 1) -> float :
         volts = self.getMaxVoltage() * value
         return volts / ((2 ** (self._adcBits - 1)) - 1)
 
@@ -284,7 +308,7 @@ class ADS1x15:
 class ADS1013(ADS1x15) :
 
     # Constructor with SMBus ID and I2C address input and configuration for ADS1013
-    def __init__(self, busId, address = I2C_address) :
+    def __init__(self, busId: int, address: int = I2C_address) :
         self.i2c = SMBus(busId)
         self._address = address
         self._conversionDelay = 2
@@ -297,7 +321,7 @@ class ADS1013(ADS1x15) :
 class ADS1014(ADS1x15) :
 
     # Constructor with SMBus ID and I2C address input and configuration for ADS1014
-    def __init__(self, busId, address = I2C_address) :
+    def __init__(self, busId: int, address: int = I2C_address) :
         self.i2c = SMBus(busId)
         self._address = address
         self._conversionDelay = 2
@@ -310,7 +334,7 @@ class ADS1014(ADS1x15) :
 class ADS1015(ADS1x15) :
 
     # Constructor with SMBus ID and I2C address input and configuration for ADS1015
-    def __init__(self, busId, address = I2C_address) :
+    def __init__(self, busId: int, address: int = I2C_address) :
         self.i2c = SMBus(busId)
         self._address = address
         self._conversionDelay = 2
@@ -321,36 +345,36 @@ class ADS1015(ADS1x15) :
 
     # Request single-shot conversion between pin 0 and pin 3
     def requestADC_Differential_0_3(self) :
-        self._requestADC(0x1000)
+        self._requestADC(1)
 
     # Get ADC value between pin 0 and pin 3
     def readADC_Differential_0_3(self) :
         self.requestADC_Differential_0_3()
-        return self.getADC()
+        return self._getADC()
 
     # Request single-shot conversion between pin 1 and pin 3
     def requestADC_Differential_1_3(self) :
-        self._requestADC(0x2000)
+        self._requestADC(2)
 
     # Get ADC value between pin 1 and pin 3
     def readADC_Differential_1_3(self) :
         self.requestADC_Differential_1_3()
-        return self.getADC()
+        return self._getADC()
 
     # Request single-shot conversion between pin 2 and pin 3
     def requestADC_Differential_2_3(self) :
-        self._requestADC(0x3000)
+        self._requestADC(3)
 
     # Get ADC value between pin 2 and pin 3
     def readADC_Differential_2_3(self) :
         self.requestADC_Differential_2_3()
-        return self.getADC()
+        return self._getADC()
 
 # ADS1113 class derifed from general ADS1x15 class
 class ADS1113(ADS1x15) :
 
     # Constructor with SMBus ID and I2C address input and configuration for ADS1113
-    def __init__(self, busId, address = I2C_address) :
+    def __init__(self, busId: int, address: int = I2C_address) :
         self.i2c = SMBus(busId)
         self._address = address
         self._conversionDelay = 8
@@ -363,7 +387,7 @@ class ADS1113(ADS1x15) :
 class ADS1114(ADS1x15) :
 
     # Constructor with SMBus ID and I2C address input and configuration for ADS1114
-    def __init__(self, busId, address = I2C_address) :
+    def __init__(self, busId: int, address: int = I2C_address) :
         self.i2c = SMBus(busId)
         self._address = address
         self._conversionDelay = 8
@@ -376,7 +400,7 @@ class ADS1114(ADS1x15) :
 class ADS1115(ADS1x15) :
 
     # Constructor with SMBus ID and I2C address input and configuration for ADS1115
-    def __init__(self, busId, address = I2C_address) :
+    def __init__(self, busId: int, address: int = I2C_address) :
         self.i2c = SMBus(busId)
         self._address = address
         self._conversionDelay = 8
@@ -387,27 +411,27 @@ class ADS1115(ADS1x15) :
 
     # Request single-shot conversion between pin 0 and pin 3
     def requestADC_Differential_0_3(self) :
-        self._requestADC(0x1000)
+        self._requestADC(1)
 
     # Get ADC value between pin 0 and pin 3
     def readADC_Differential_0_3(self) :
         self.requestADC_Differential_0_3()
-        return self.getADC()
+        return self._getADC()
 
     # Request single-shot conversion between pin 1 and pin 3
     def requestADC_Differential_1_3(self) :
-        self._requestADC(0x2000)
+        self._requestADC(2)
 
     # Get ADC value between pin 1 and pin 3
     def readADC_Differential_1_3(self) :
         self.requestADC_Differential_1_3()
-        return self.getADC()
+        return self._getADC()
 
     # Request single-shot conversion between pin 2 and pin 3
     def requestADC_Differential_2_3(self) :
-        self._requestADC(0x3000)
+        self._requestADC(3)
 
     # Get ADC value between pin 2 and pin 3
     def readADC_Differential_2_3(self) :
         self.requestADC_Differential_2_3()
-        return self.getADC()
+        return self._getADC()
